UNIT uMAListe;
(* ********************************************************************  *)
(*                                                                       *)
(* K L A S S E   : TMaListe                                              *)
(* --------------------------------------------------------------------  *)
(* Version       : 1.                                                    *)
(* Autor         : Bjrn Spiegel, Maria Welle, OSZ-Handel I, 10997 Berlin*)
(*                                                                       *)
(* Aufgabe       : Dynamische Liste zur Verwaltung der Mitarbeiter-      *)
(*                 Objekte.                                              *)
(*                                                                       *)
(* Compiler      : Delphi 4.0                                            *)
(* Aenderung     : V. 1.   - 29-11-2000                                  *)
(*                           21-12-2000                                  *)
(* ********************************************************************  *)

INTERFACE
(* ===================================================================== *)
USES  Classes,
      Sysutils,
      uDListe,
      uMitarbeiter;


type
  TElement = TMitarbeiter;

  TMaListe  = class (TListe)
         public
          constructor Create;
          function  SearchElement (suchElem : TElement) : TElement;
          procedure Load (Dateiname : string); override;
          procedure Store(Dateiname : string); override;
         end;

(*  ----------------- B e s c h r e i b u n g -------------------------

Oberklasse     : TListe
Bezugsklassen  : TMitarbeiter
                 Classes - import: TFilstream, read, write
                 Sysutils        : Fileexists
Methoden
--------

Create
  Auftrag: Mitarbeiterliste erzeugen
  vorher : -
  nachher: ist vorhanden; alle Felder sind leer

SearchElement (suchElem : TElement) : TElement;
  Anfrage: ob suchElem in der Liste gefunden wurde
  vorher : -
  nachher: Gefundenes Element wird zurckgegeben. Wenn Liste leer ist oder
           nicht gefunden wird EoList zurckgegeben.


Load (dateiname : string);
  Auftrag: Liste aus externer Datei laden. Wenn keine Datei existiert,
           dann neue Datei mit der Laenge 0 anlegen.
  vorher : Die Liste ist initialisiert.
  nachher: Die Liste ist in den Arbeitsspeicher geladen. Datei ist geschlossen.


Store (dateiname : string);
  Auftrag: Liste in externe Datei speichern
  vorher : -
  nachher: Die Liste ist gespeichert. Ist die Liste leer, wird nichts
          gespeichert. Datei ist geschlossen.

----------------------------------------------------------------------- *)


IMPLEMENTATION
(* ==================================================================== *)


constructor TMaliste.Create;
begin
  inherited Create;
  Init;
end;

function  TMaListe.SearchElement (suchElem : TElement) : TElement;
(* -------------------------------------------------------------------- *)
var gefunden : boolean;
    Elem     : TElement;

begin
  Result   := NIL;
  gefunden := false;
  First;

  while NOT (gefunden or Eolist) do
  begin
    Elem := GetElement;
    if Elem.IstGleich(suchElem)
    then
      begin
        gefunden := true;
        Result := Elem;
      end
    else Next;
  end;
end;


procedure TMaListe.Load (Dateiname : string);
(* -------------------------------------------------------------------- *)
var
    Elem  : TElement;
    Fs    : TFilestream;
    i,
    Anzahl: integer;

begin

  Anzahl := 0 ;
  if NOT FileExists(Dateiname)
  then Fs := TFilestream.Create(Dateiname, fmCreate)
  else Fs := TFilestream.Create(Dateiname, fmOpenRead);

  if Fs.Size > 0
  then
    begin
      Fs.Read(Anzahl, SizeOf(LiLaenge));
      for i := 1 to Anzahl do
      begin
        elem := Telement.Create;
        Elem.AusDateiLesen(Fs);
        Append(Elem);
      end;
      First;
    end;
  Fs.Free;
end;


procedure TMaListe.Store(Dateiname : string);
(* -------------------------------------------------------------------- *)
var Elem  : TElement;
    Fs    : TFilestream;

begin
  DeleteFile(Dateiname);
  Fs := TFilestream.Create(Dateiname, fmCreate);

  if not IsEmpty
  then
  begin
    Fs.Write(LiLaenge, SizeOf(LiLaenge));
    First;
    while NOT EoList do
    begin
      Elem := GetElement;
      Elem.InDateiSchreiben (Fs);
      Next;
    end;
  end;
  Fs.Free;
end;

END.
