UNIT uMitarbeiter;

(* ******************************************************************** *)
(* K L A S S E : TMitarbeiter                                           *)
(* -------------------------------------------------------------------- *)
(* Version     : 1.31                                                   *)
(* Autor       : Bjrn Spiegel, Maria Welle, OSZ-Handel I, 10997 Berlin *)
(*                                                                      *)
(* Beschreibung: bildet das Objekt Mitarbeiter ab.                      *)
(*                                                                      *)
(* Compiler    : DELPHI 4.0                                             *)
(* Aenderung   : V. 1. 29.11.2000                                       *)
(*                     21-12-2000                                       *)       
(* ******************************************************************** *)

INTERFACE
(* ========================== Export ================================== *)
USES  Classes,
      uPerson,
      uAdresse;

type
   TMitarbeiter = class (TPerson)
      Adresse        : TAdresse;
    private
      Status         : 0..3   ;  (* verfuegbar, frei, Urlaub, fehlt*)
      Persnr         : integer;
      Kueche         : boolean;
      Bueffet        : boolean;
      Spuele         : boolean;
      Frueh          : boolean;
      Spaet          : boolean;
      Montag         : boolean;
      Dienstag       : boolean;
      Mittwoch       : boolean;
      Donnerstag     : boolean;
      Freitag        : boolean;
      Samstag        : boolean;
      Sollstunden    : real;
      Iststunden     : real;
      Arbeitstage    : array[1..6] of boolean;  // soll verhindern, da Mitarbeiter an gleichen tagen eingesetzt werden

    public
      constructor Create;                       override;
      procedure Init;                           override;
      procedure SetStatus     (st  : byte);     virtual;
      procedure SetKueche     (k   : boolean);  virtual;
      procedure SetBueffet    (b   : boolean);  virtual;
      procedure SetSpuele     (s   : boolean);  virtual;
      procedure SetPersnr     (pnr : integer);  virtual;
      procedure SetFrueh      (f   : boolean);  virtual;
      procedure SetSpaet      (sp  : boolean);  virtual;
      procedure SetMontag     (mot : boolean);  virtual;
      procedure SetDienstag   (dit : boolean);  virtual;
      procedure SetMittwoch   (mit : boolean);  virtual;
      procedure SetDonnerstag (dot : boolean);  virtual;
      procedure SetFreitag    (frt : boolean);  virtual;
      procedure SetSamstag    (sat : boolean);  virtual;
      procedure SetSollstunden(soll: real);  virtual;
      procedure SetIststunden (ist : real);  virtual;
      procedure IststundenErhoehenUm (um : real);
      procedure SetTagBesetzt(tag : integer);
      procedure SetAlleTageFrei;

      function GetStatus         : byte;           virtual;
      function GetFrueh          : boolean;        virtual;
      function GetSpaet          : boolean;        virtual;
      function GetPersnr         : integer;        virtual;
      function GetKueche         : boolean;        virtual;
      function GetBueffet        : boolean;        virtual;
      function GetSpuele         : boolean;        virtual;
      function GetMontag         : boolean;        virtual;
      function GetDienstag       : boolean;        virtual;
      function GetMittwoch       : boolean;        virtual;
      function GetDonnerstag     : boolean;        virtual;
      function GetFreitag        : boolean;        virtual;
      function GetSamstag        : boolean;        virtual;
      function GetSollstunden    : real;        virtual;
      function GetIststunden     : real;        virtual;
      function ArbeitetIn   (ab  : string)   : boolean;
      function IstVerfuegbar(tag : integer)  : boolean;
      //function IstGleich (ma : TMitarbeiter) : boolean; reintroduce; overload;

      procedure InDateiSchreiben (var Fs : TFilestream); override;
      procedure AusDateiLesen    (var Fs : TFilestream); override;
    end; (* TMitarbeiter *)

(* -------------------- B e s c h r e i b u n g -------------------------

Oberklasse     : TPerson
Bezugsklassen  : TClasses - import: TFilestream, read,write


Methoden
--------

Create
  Auftrag: Mitarbeiteren erzeugen
  vorher : -
  nachher: ist vorhanden; alle Felder sind leer

Init
  Auftrag: Mitarbeiteren Initialisieren
  vorher : Person ist vorhanden.
  nachher: alle Felder sind leer

Set...
  Auftrag: Attribut schreiben
  vorher : Person ist vorhanden.
  nachher: Attribut ist gesetzt

Get...
  Auftrag: Attribut aus dem Objekt lesen
  vorher : Person ist vorhanden.
  nachher: -

IstVerfuegbar(tag : integer)  : boolean
  Anfrage: pruefen, ob Mitarbeiter an tag anwesend ist und
           den Status 0 hat und noch Sollstunden verfgbar hat.
  vorher : Mitarbeiter ist vorhanden.
  nachher: Mitarbeiter ist entweder verfgbar oder nicht

function ArbeitetIn   (ab  : string)   : boolean
  Anfrage: pruefen, in welchem Bereich (Spuele, Bueffet, Kueche) der Mitarbeiter
           einsetzbar ist
  vorher : Mitarbeiter ist vorhanden.
  nachher: Mitarbeiter ist in einem Bereich einsetzbar oder nicht

IstGleich (suchpat : TMitarbeiter) : boolean
  Anfrage: pruefen, ob Mitarbeiter gleich suchpat ist.
  vorher : Mitarbeiter ist initialisiert.
  nachher: IstGleich ist wahr, wenn beide Namen uebereinstimmen.

InDateiSchreiben (var fs : TFilestream)
  Auftrag: Attribute auf die Festplatte speichern
  vorher : fs ist geffnet
  nachher: Attribute sind an fs uebergeben


AusDateiLesen (var fs : TFilestream)
  Auftrag: Attribute von der Festplatte laden
  vorher : fs ist geffnet
  nachher: Attribute sind an Mitarbeiter uebergeben
----------------------------------------------------------------------- *)

IMPLEMENTATION
(* ==================================================================== *)

constructor TMitarbeiter.Create;
begin
  inherited Create;
  Adresse := TAdresse.Create;
  Init;
end;


procedure TMitarbeiter.Init;
(* -------------------------------------------------------------------- *)
var i : integer;
begin
  inherited Init;   // was von TPerson kommt
	Status       := 0;
	Persnr       := 0;
	Kueche       := false;
  Spuele       := false;
  Bueffet      := false;
  Frueh        := false;
  Spaet        := false;
  Sollstunden  := 0;
  Iststunden   := 0;
  Montag       := false;
  Dienstag     := false;
  Mittwoch     := false;
  Donnerstag   := false;
  Freitag      := false;
  Samstag      := false;

  for i := 1 to 6 do
    Arbeitstage[i] := false;
end;


procedure TMitarbeiter.SetPersnr (pnr : integer);
(* -------------------------------------------------------------------- *)
begin
  Persnr := pnr;
end;


procedure TMitarbeiter.SetFrueh (f : boolean);
(* -------------------------------------------------------------------- *)
begin
  Frueh := f;
end;

procedure TMitarbeiter.SetSpaet (sp : boolean);
(* -------------------------------------------------------------------- *)
begin
  Spaet := sp;
end;

procedure TMitarbeiter.SetSpuele (s : boolean);
(* -------------------------------------------------------------------- *)
begin
  Spuele := s;
end;

procedure TMitarbeiter.SetBueffet (b : boolean);
(* -------------------------------------------------------------------- *)
begin
  Bueffet := b;
end;

procedure TMitarbeiter.SetKueche (k : boolean);
(* -------------------------------------------------------------------- *)
begin
  Kueche := k;
end;

procedure TMitarbeiter.SetMontag (mot : boolean);
(* -------------------------------------------------------------------- *)
begin
  Montag := mot;
end;

procedure TMitarbeiter.SetDienstag (dit : boolean);
(* -------------------------------------------------------------------- *)
begin
  Dienstag := dit;
end;

procedure TMitarbeiter.SetMittwoch (mit : boolean);
(* -------------------------------------------------------------------- *)
begin
  Mittwoch := mit;
end;

procedure TMitarbeiter.SetDonnerstag (dot : boolean);
(* -------------------------------------------------------------------- *)
begin
  Donnerstag := dot;
end;

procedure TMitarbeiter.SetFreitag (frt : boolean);
(* -------------------------------------------------------------------- *)
begin
  Freitag := frt;
end;

procedure TMitarbeiter.SetSamstag (sat : boolean);
(* -------------------------------------------------------------------- *)
begin
  Samstag := sat;
end;

procedure TMitarbeiter.SetSollstunden (soll : real);
(* -------------------------------------------------------------------- *)
begin
  Sollstunden := soll;
end;

procedure TMitarbeiter.IststundenErhoehenUm (um : real);
(* -------------------------------------------------------------------- *)
begin
  IstStunden := IstStunden + um;
end;

procedure TMitarbeiter.SetIststunden (ist : real);
(* -------------------------------------------------------------------- *)
begin
  Iststunden := ist;
end;

procedure TMitarbeiter.SetStatus(st : byte);
(* -------------------------------------------------------------------- *)
begin
  Status := st;
end;

procedure TMitarbeiter.SetTagBesetzt;
(* -------------------------------------------------------------------- *)
begin
     Arbeitstage[tag] := true;
end;

procedure TMitarbeiter.SetAlleTageFrei;
(* -------------------------------------------------------------------- *)
var i : integer;
begin
  for i:= 1 to 6 do
    Arbeitstage[i] := false;
end;

function TMitarbeiter.GetStatus : byte;
(* -------------------------------------------------------------------- *)
begin
 result := Status;
end;


function TMitarbeiter.GetPersnr : integer;
(* -------------------------------------------------------------------- *)
begin
  result := Persnr;
end;


function TMitarbeiter.GetSpuele : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Spuele;
end;


function TMitarbeiter.GetKueche : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Kueche;
end;

function TMitarbeiter.GetBueffet : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Bueffet;
end;

function TMitarbeiter.GetFrueh : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Frueh;
end;

function TMitarbeiter.GetSpaet : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Spaet;
end;

function TMitarbeiter.GetMontag : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Montag;
end;

function TMitarbeiter.GetDienstag : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Dienstag;
end;

function TMitarbeiter.GetMittwoch : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Mittwoch;
end;

function TMitarbeiter.GetDonnerstag : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Donnerstag;
end;

function TMitarbeiter.GetFreitag : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Freitag;
end;

function TMitarbeiter.GetSamstag : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := Samstag;
end;
{
function TMitarbeiter.IstGleich (ma : TMitarbeiter) : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := inherited IstGleich(ma);
end;
}

function TMitarbeiter.GetSollstunden : real;
(* -------------------------------------------------------------------- *)
begin
  result := Sollstunden;
end;

function TMitarbeiter.GetIststunden : real;
(* -------------------------------------------------------------------- *)
begin
  result := IstStunden;
end;

function TMitarbeiter.ArbeitetIn (ab : string) : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := false;

  if  (ab =  'Kche') and (Kueche = true)
  then result := true;

  if  (ab =  'Bffet') and (Bueffet = true)
  then result := true;

  if  (ab =  'Sple') and (Spuele = true)
  then result := true;

  if result = true then ab := '';
end;

function TMitarbeiter.IstVerfuegbar (tag : integer) : boolean;
(* -------------------------------------------------------------------- *)
begin
  result := false;

  if (tag = 1)
   and (Montag = true)
   and (Status =0)
   and (Sollstunden - Iststunden > 0)
  then result := true;

  if (tag = 2)
   and (Dienstag = true)
   and (Status =0)
   and (Sollstunden - Iststunden > 0)
  then result := true;

  if (tag = 3)
   and (Mittwoch = true)
   and (Status =0)
   and (Sollstunden - Iststunden > 0)
  then result := true;

  if (tag = 4)
   and (Donnerstag = true)
   and (Status =0)
   and (Sollstunden - Iststunden > 0)
  then result := true;

  if (tag = 5)
   and (Freitag = true)
   and (Status =0)
   and (Sollstunden - Iststunden > 0)
  then result := true;

  if (tag = 6)
   and (Samstag = true)
   and (Status =0)
   and (Sollstunden - Iststunden > 0)
  then result := true;

  if Arbeitstage[tag] then result := false;
end;

procedure TMitarbeiter.InDateiSchreiben (var Fs : TFilestream);
(* -------------------------------------------------------------------- *)
begin
  inherited InDateiSchreiben(Fs);
  Fs.write( Status, sizeoF(Status));
  Fs.write( Persnr, sizeoF(Persnr));
  Fs.write( Kueche, sizeoF(Kueche));
  Fs.write( Bueffet, sizeoF(Bueffet));
  Fs.write( Spuele, sizeoF(Spuele));
  Fs.write( Frueh, sizeoF(Frueh));
  Fs.write( Spaet, sizeoF(Spaet));
  Fs.write( Montag, sizeoF(Montag));
  Fs.write( Dienstag, sizeoF(Dienstag));
  Fs.write( Mittwoch, sizeoF(Mittwoch));
  Fs.write( Donnerstag, sizeoF(Donnerstag));
  Fs.write( Freitag, sizeoF(Freitag));
  Fs.write( Samstag, sizeoF(Samstag));
  Fs.write( Sollstunden, sizeoF(Sollstunden));
  Fs.write( Iststunden, sizeoF(Iststunden));
  Adresse.InDateiSchreiben(fs);

end;


procedure TMitarbeiter.AusDateiLesen (var Fs : TFilestream) ;
(* -------------------------------------------------------------------- *)
begin
  inherited AusDateiLesen(Fs);
  Fs.read( Status, sizeoF(Status));
  Fs.read( Persnr, sizeoF(Persnr));
  Fs.read( Kueche, sizeoF(Kueche));
  Fs.read( Bueffet, sizeoF(Bueffet));
  Fs.read( Spuele, sizeoF(Spuele));
  Fs.read( Frueh, sizeoF(Frueh));
  Fs.read( Spaet, sizeoF(Spaet));
  Fs.read( Montag, sizeoF(Montag));
  Fs.read( Dienstag, sizeoF(Dienstag));
  Fs.read( Mittwoch, sizeoF(Mittwoch));
  Fs.read( Donnerstag, sizeoF(Donnerstag));
  Fs.read( Freitag, sizeoF(Freitag));
  Fs.read( Samstag, sizeoF(Samstag));
  Fs.read( Sollstunden, sizeoF(Sollstunden));
  Fs.read( Iststunden, sizeoF(Iststunden));
  Adresse.AusDateiLesen(fs);
end;


END.
