unit uPool;
    (* ******************************************************************** *)
    (*                                                                      *)
    (* K L A S S E : TSchicht                                               *)
    (* -------------------------------------------------------------------- *)
    (* Version     : 1.0                                                    *)
    (* Autor       : Martin Kcher, Norman Niemer                           *)
    (*                                                                      *)
    (* Aufgabe     : Erstellt einen Mitarbeiter-Pool.                       *)
    (*                                                                      *)
    (* Compiler    : DELPHI 4.0                                             *)
    (* Aenderung   : 8.11.00 Anfang                                         *)
    (*              11.11.00 Struktur fertig                                *)
    (*              19.12.00 NN: Remove raus, da Korrektur d. IstStunden    *)
    (*                           in TSchicht                                *)
    (*              26.12.00 NN: NextMA, Auffuellen, SetAktPos verndert    *)
    (*                                                                      *)
    (*                                                                      *)
    (* ******************************************************************** *)
INTERFACE
(* ================================ Export ================================== *)
USES uMAListe, uMitarbeiter, Classes,
     {Debug:} Sysutils, Dialogs;

type
   TPool = class (TMAListe)
      private

      public
        constructor Create;
        
        procedure SetAktuellePos(AktPos : integer);

        function Auffuellen(ma : TMAListe; tag : integer) : boolean;
        function NextMA : boolean;
        function PrevMA : boolean;
   end; (* TPool *)

(* ---------------------- B e s c h r e i b u n g ------------------------------

Oberklassen   : TMAListe
Bezugsklassen : import: TMitarbeiter
                export: TSchicht

Methoden
--------

Create
  Auftrag: Pool erzeugen
  vorher : ---
  nachher: Pool ist erzeugt

Free
  Auftrag: loescht das Objekt
  vorher : Pool ist vorhanden
  nachher: Liste ist geloescht und Speicher freigegeben

Remove (ma : TMitarbeiter)
  Auftrag: loescht den Eintrag des Mitarbeiters aus dem Pool
           Ist-Stunden wird in der MAListe um 0.5 Stunden erhht
  vorher : Pool gefuellt & Mitarbeiter eingetragen
  nahcher: ---

SetAktuellePos (ActPos : integer)
  Auftrag: setzt die Listenposition auf das gewuenschte Element
  vorher : Pool ist nicht leer
  nachher: Listenposition auf gewuenschtem Element

Auffuellen (ma : TMAListe)
  Auftrag: traegt die Mitarbeiter fuer die jeweilige maximale Anzahl an
           Arbeitsbloecken fuer alle Tage, alle Arbeitsplaetze und alle
           Schichten in den Pool ein
  vorher : Pool ist erzeugt
  nachher: Pool ist gefuellt

NextMA
  Auftrag: setzt den Listenzeiger auf den Eintrag mit einer neuen
           Mitarbeiternummer (da ein Mitarbeiter x-mal vorhanden ist)
  vorher : Pool ist vorhanden & nicht leer
  nachher: Listenzeiger auf dem naechsten Mitarbeiter
  Rueckgabewert: true  = Zeiger gesetzt
                 false = Pool ist leer oder nur 1 Mitarbeiter vorhanden

PrevMA
  Auftrag: analog zu NextMA
----------------------------------------------------------------------------- *)

IMPLEMENTATION
(* ========================================================================== *)

constructor TPool.Create;
(* -------------------------------------------------------------------------- *)
begin
   inherited Create;
end;

procedure TPool.SetAktuellePos;
(* -------------------------------------------------------------------------- *)
begin
     if not(aktpos >= LiLaenge) then AktuellePos := aktpos;
end;

function TPool.Auffuellen;
(* -------------------------------------------------------------------------- *)
var i, maxbloecke : integer;
    mitarb : TMitarbeiter;
begin
   Result := False;
   ma.First;
   while not ma.EoList do
   begin
      mitarb := ma.getElement;
      if mitarb.IstVerfuegbar(tag)
      then begin
         maxbloecke :=  round(mitarb.getSollStunden - mitarb.getIstStunden) * 2;
         For i :=1 to maxbloecke do
            append(mitarb);
      end;
      ma.Next;
   end;
   ma.First;
   Result := true;
end;

function TPool.NextMA : boolean;
(* -------------------------------------------------------------------------- *)
var mitarb1, mitarb2 : TMitarbeiter;
    abbruch : boolean;
begin
   mitarb1 := TMitarbeiter.Create;
   mitarb2 := TMitarbeiter.Create;

   Result := false;
   abbruch := false;
   if not IsEmpty then
   begin
           mitarb1 := getElement;
           Next;
           while not abbruch do
           begin
              mitarb2 := getElement;
              if mitarb2 <> nil
              then
                   if mitarb2.getPersnr = mitarb1.getPersnr
                   then
                        if eoList then
                           abbruch := true
                        else Next
                   else
                   begin
                        Result := True;
                        abbruch := true;
                   end
              else abbruch := true;
           end;
   end;
end;

function TPool.PrevMA : boolean;
(* -------------------------------------------------------------------------- *)
var mitarb1, mitarb2 : TMitarbeiter;
    abbruch : boolean;
begin
   mitarb1 := TMitarbeiter.Create;
   mitarb2 := TMitarbeiter.Create;

   Result := false;
   abbruch := false;
   if not IsEmpty then
   begin
    mitarb1 := getElement;
    Next;
    while not abbruch do
    begin
      mitarb2 := getElement;
      if mitarb2 <> nil
      then
        if mitarb2.getPersnr = mitarb1.getPersnr
        then
          if eoList
          then abbruch := true
          else Previous
            else
              begin
                Result := True;
                abbruch := true;
              end
        else abbruch := true;
      end;
   end;
end;
// =============================================================================
end. // ================== E N D    U N I T    u P o o l    ====================
