UNIT uSchicht;

(* ******************************************************************** *)
(*                                                                      *)
(* K L A S S E : TSchicht                                               *)
(* -------------------------------------------------------------------- *)
(* Version     : 1.0                                                    *)
(* Autor       : Fabian Trauptmann, Norman Niemer (Besetzen, MAPasst)   *)
(*                                                                      *)
(* Aufgabe     : verwaltet eine Schicht mit den ein Mitarbeitern        *)
(*                                                                      *)
(* Compiler    : DELPHI 4.0                                             *)
(* Aenderung   : 8.11.00 Anfang                                         *)
(*              11.11.00 Struktur fertig                                *)
(*              15.11.00 Vererbung von TObject, Besetzung als TPool     *)
(*              30.11.00 NN: Implementation MAPasst, Besetzen           *)
(*              16.12.00 NN: Besetzen fertig aber nicht getestet        *)
(*              26.12.00 NN: Fehlerbehebung                             *)
(*                                                                      *)
(* ******************************************************************** *)

INTERFACE
(* ========================== Export ================================== *)
USES Classes,
  uPool, uMitarbeiter, uMAListe;

type
   TSchicht = class (TObject)
     private
       Anfang, Ende,
       art : integer;       // 0 = Frhschicht, 1 = Sptschicht
       Arbeitsplatz : string[20];

       function BlockBesetzen(MA : TMitarbeiter; block : integer) : boolean;               virtual;
       function MAPasst ( MA : TMitarbeiter; Tag : integer) : boolean;
     public
       Besetzung : TPool;

       constructor Create(Aplatz : string; p_anf : integer; p_ende : integer; p_art : integer); virtual;
       destructor Free;
       procedure SchichtFreigeben;                       virtual;
       procedure BlockFreibegen(block : integer);        virtual;
       procedure SetAnfang(anf : integer);               virtual;
       procedure SetEnde(e : integer);                   virtual;
       procedure SetArbeitsplatz(platz : string);        virtual;

       function Besetzen(Tag : integer; var pool : TPool; var MAListe : TMAListe) : boolean; virtual;
       function GetBesetzung : TPool;                    virtual;
       function GetAnfang : integer;                     virtual;
       function GetEnde : integer;                       virtual;
       function GetArbeitsplatz : string;                virtual;
       function GetArt          : integer;               virtual;
       function GetAnzahlBloecke : integer;              virtual;

       procedure InDateiSchreiben (var fs : TFilestream); virtual;
       procedure AusDateiLesen    (var fs : TFilestream); virtual;
     end; (* TSchicht *)

(* -------------------- B e s c h r e i b u n g -------------------------

Oberklasse     : TObject
Bezugsklassen  : import: TMitarbeiter, TPool
                 export: TTag,
                         TFensterFrm


Methoden
--------

Create (Arbeitsplatz, Anfang, Ende)
  Auftrag: Schicht erzeugen
  vorher : -
  nachher: Schicht ist erzeugt

Set...
  Auftrag: Attribut schreiben
  vorher : Schicht ist vorhanden.
  nachher: Attribut ist gesetzt

Get...
  Auftrag: Attribut aus dem Objekt lesen
  vorher : Schicht ist vorhanden.
  nachher: -

Besetzen
  Auftrag: versucht die Schicht mit Mitarbeitern zu besetzen
  vorher : Schicht mu initialisiert sein
  nachher: Schicht ist aufgefllt
  Rckgabewerte: true = erfolgreich eingetragen
                 false = keine Besetzung mglich
                 
SchichtBesetzen
  Auftrag: trgt in der ganzen Schicht die Mitarbeiter ein
           und loescht den entsprechenden Eintrag im Pool
  vorher : Schicht mu initialisiert sein & Pool aufgefllt
  nachher: Schicht ist aufgefllt


SchichtFreigeben
  Auftrag: entfernt den Mitarbeiter aus der Schicht
           und trgt ihn im Pool wieder ein
  vorher : Mitarbeiter ist in der Schicht vorhanden
  nachher: Mitarbeiter ist aus der Schicht entfernt und im Pool wieder vorhanden

BlockBesetzen
  Auftrag: trgt in einem Block den Mitarbeiter ein
           und loescht den entsprechenden Eintrag im Pool
  vorher : Block mu initialisiert sein und Pool aufgefllt
  nachher: Block ist aufgefllt

BlockFreigeben
  Auftrag: entfernt den Mitarbeiter aus dem Block
           und trgt ihn im Pool wieder ein
  vorher : Mitarbeiter ist in dem Block vorhanden
  nachher: Mitarbeiter ist aus dem Block entfernt und im Pool wieder vorhanden

GetArt
  Auftrag: ermittelt, ob die Schicht eine Frh- oder Sptschicht ist
  vorher : -
  nachher: -

MAPasst
  Auftrag: prft, ob der Mitarbeiter in der Schicht einsetzbar ist
  Rckgabewerte: true = einsetzbar
                 false = nicht einsetzbar

Free
  Auftrag: loescht die Schicht und evtl. den Pool
  vorher : Schicht ist vorhanden.
  nachher: Objekt Schicht & Pool sind geloescht

----------------------------------------------------------------------- *)

IMPLEMENTATION
(* ==================================================================== *)

constructor TSchicht.Create;
(* -------------------------------------------------------------------- *)
begin
   inherited Create;

   Arbeitsplatz := Aplatz;
   anfang := p_anf;
   ende := p_ende;
   art := p_art;
   Besetzung := TPool.Create;
end;

destructor TSchicht.Free;
(* -------------------------------------------------------------------- *)
begin
  Besetzung.Free;
end;

function TSchicht.MAPasst;
(* -------------------------------------------------------------------- *)
var s : string;
    _pos : integer;
begin
     Result := false;

     _pos := pos(' ', Arbeitsplatz);  // alle Arbeitspltze heissen z.B. 'Kche 1' oder 'Sple 2' -> vorbereiten fr ArbeitetIn
     s := copy(Arbeitsplatz, 1, _pos-1);

     if art = 0 then           // Frhsschicht?
     begin
        if MA.ArbeitetIn(s) and
           MA.IstVerfuegbar(Tag) and // prft, ob der MA heute schon eingesetzt wurde
           MA.GetFrueh
        then Result := true
     end
     else                     // oder Sptschicht
     begin
         if MA.ArbeitetIn(s) and
            MA.IstVerfuegbar(Tag) and
            MA.GetSpaet then
            Result := true;
     end;
end;

function TSchicht.BlockBesetzen;
(* -------------------------------------------------------------------- *)
var pos : integer;
begin
     pos := Besetzung.GetAktuellePos;
     Besetzung.SetAktuellePos(block);
     Besetzung.PutElement(ma);
     Besetzung.SetAktuellePos(pos);
     Result := true;
end;

function TSchicht.Besetzen;
var ma : TMitarbeiter;
    i, pos,
    anzahlbloecke,
    bloeckebesetzt : integer;
    sinnlos,                       // es konnte keine Besetzung gefunden werden
    allesbesetzt,                  // alle bloecke erfolgreich besetzt
    alledurchlaufen : boolean;     // soll prfen, ob pool schon einmal durchsucht, dann nmlich unterschiedliche MA fr Schicht
begin
     Result := false;

     ma := TMitarbeiter.Create;
     anzahlbloecke := GetAnzahlBloecke;
     bloeckebesetzt := 0;
     sinnlos         := false;
     allesbesetzt    := false;
     alledurchlaufen := false;

     pool.First;
     Besetzung.RemoveAll;   // es wird immer ein komplett neue Besetzung erstellt

     if not pool.EoList then
     begin
          repeat
                ma := pool.GetElement;
                if MAPasst(ma, tag) then
                begin
                     if ((ma.getSollStunden - ma.getIstStunden) * 2) >= anzahlbloecke then
                     begin
                          pos := pool.GetAktuellePos;
		          for i:= 1 to anzahlbloecke do
                          begin
                               Besetzung.Append(ma);
                               inc(bloeckebesetzt);
                               ma := MAListe.SearchElement(ma);
                               ma.IstStundenErhoehenUm(0.5);
                               pool.Remove;
                               pool.SetAktuellePos(pos);
                               ma := pool.GetElement;
                          end;
                          ma.SetTagBesetzt(tag);
                          allesbesetzt := true;
                     end
                     else                                  // nicht fr gesamten Block
                         if alledurchlaufen then
                         begin
                              for i:= 1 to round((ma.getSollStunden - ma.getIstStunden) * 2) do               //!!!!MA.Restbloecke
                              begin
                                   Besetzung.Append(ma);
                                   inc(bloeckebesetzt);
                                   ma := MAListe.SearchElement(ma);
                                   ma.IstStundenErhoehenUm(0.5);
                                   pos := pool.GetAktuellePos;
                                   pool.Remove;
                                   pool.SetAktuellePos(pos);
                                   if bloeckebesetzt >= anzahlbloecke then
                                   begin
                                        allesbesetzt := true;
                                        break;               // !!! bricht for-Schleife ab? oder nur aktuellen durchlauf?
                                   end
                                   else
                                       if pool.EoList then
                                       begin
                                            sinnlos := true;
                                            break;
                                       end
                                       else ma := pool.GetElement;
                              end;
                         end
                     else
                         if not pool.NextMA then
                         begin
                              alledurchlaufen := true;
                              pool.First;
                         end;
                end
                else
                    if not pool.NextMA then sinnlos := true;

          until allesbesetzt or sinnlos;
     end;

     if allesbesetzt then Result := true;
     Besetzung.First;
end;

procedure TSchicht.SetAnfang;
(* -------------------------------------------------------------------- *)
begin
  Anfang := anf;
end;


procedure TSchicht.SetEnde;
(* -------------------------------------------------------------------- *)
begin
  Ende := e;
end;


procedure TSchicht.SetArbeitsplatz;
(* -------------------------------------------------------------------- *)
begin
  Arbeitsplatz := platz;
end;


function TSchicht.GetBesetzung;
(* -------------------------------------------------------------------- *)
begin
  result := Besetzung;
end;


function TSchicht.GetAnfang;
(* -------------------------------------------------------------------- *)
begin
  result := Anfang;
end;

function TSchicht.GetArt;
(* -------------------------------------------------------------------- *)
begin
  result := art;
end;

function TSchicht.GetEnde;
(* -------------------------------------------------------------------- *)
begin
  result := Ende;
end;


function TSchicht.GetArbeitsplatz;
(* -------------------------------------------------------------------- *)
begin
  result := Arbeitsplatz;
end;

procedure TSchicht.SchichtFreigeben;
begin
     Besetzung.RemoveAll;
end;

procedure TSchicht.BlockFreibegen(block : integer);
(* -------------------------------------------------------------------- *)
var pos : integer;
begin
     pos := Besetzung.GetAktuellePos;
     if block = pos then Besetzung.Remove
     else
     begin
         Besetzung.SetAktuellePos(block);
         Besetzung.Remove;
         Besetzung.SetAktuellePos(pos);
     end;
end;

function TSchicht.GetAnzahlBloecke : integer;
begin
     Result := (Ende - Anfang)+1;
end;

procedure TSchicht.InDateiSchreiben (var Fs : TFileStream);
(* -------------------------------------------------------------------- *)
begin
  Fs.write(Anfang, sizeof(Anfang));
  Fs.write(Ende, sizeof(Ende));
  Fs.write(Arbeitsplatz, sizeof(Arbeitsplatz));
end;


procedure TSchicht.AusDateiLesen (var Fs : TFileStream) ;
(* -------------------------------------------------------------------- *)
begin
  Fs.read(Anfang, sizeof(Anfang));
  Fs.read(Ende, sizeof(Ende));
  Fs.read(Arbeitsplatz, sizeof(Arbeitsplatz));
end;


END.
