unit uTag;

(* ******************************************************************** *)
(*                                                                      *)
(* K L A S S E : TTag                                                   *)
(* -------------------------------------------------------------------- *)
(* Version     : 1.0                                                    *)
(* Autor       : Norman Niemer                                          *)
(*                                                                      *)
(* Aufgabe     : verwaltet alle Schichten eines Tages fr mehrere       *)
(*               Arbeitspltze                                          *)
(*                                                                      *)
(* Compiler    : DELPHI 4.0                                             *)
(* Aenderung   : 8.11.00 Anfang                                         *)
(*              11.11.00 Struktur fertig                                *)
(*              15.11.00 Schichten als TDListe                          *)
(*              20.11.00 Implementation                                 *)
(*              16.12.00 Implementation fertig,                         *)
(*                       Spezifikation aktualisiert                     *)
(*              19.12.00 MAListe als Parameter von Besetzen             *)
(*                       Pool wird bei jedem Besetzenaufruf neu gefllt *)
(*              20.12.00 Fehlerbehebung                                 *)
(* ******************************************************************** *)

interface
(* ========================== Export ================================== *)
uses Classes, uSchicht, uDListe, uMAListe, uPool, sysutils {inttostr};

type
   TTag = class (TObject)
     private
       Datum        : string[10];
       Tag,
       AnzahlAP,
       anzBueffet,
       anzKueche,
       anzSpuele     : integer;

       pool : TPool;             // Pool von verfgbaren Mitarbeitern

     public
       Schichten : TListe;  // 1. Eintrag Frhschicht, 2. Eintragsptschicht,
                            //  3. Eintrag Frhschicht 2. Arbeitsplatz, usw.
       constructor Create(WTag: integer; dat : string;
                            Arbeitsplatz : string; EndeFS: integer;
                            EndeSS: integer); virtual;
       destructor  Free;

       function Besetzen( MAListe : TMAListe) : boolean;                  virtual;
       function GetWochentag                  : integer;           virtual;
       function GetDatum                      : string;            virtual;
       function GetAnzahlAP                   : integer;           virtual;
       function GetSchichten                  : TListe;
       procedure NeuerArbeitsplatz(Arbeitsplatz : string; EndeFS: integer;
                                    EndeSS: integer ); virtual;
       procedure SetWochentag (WTag: integer);      virtual;
       procedure SetDatum (d: string);             virtual;

       procedure InDateiSchreiben (var fs : TFilestream); virtual;
       procedure AusDateiLesen    (var fs : TFilestream); virtual;
    end; (* TTag *)

(* -------------------- B e s c h r e i b u n g -------------------------

Oberklasse     : -
Bezugsklassen  :  import: TSchicht, TPool
                  export: TWoche,
                          TFensterFrm


Methoden
--------

Create
  Auftrag: Tag sowie 1. Frh- und 1. Sptschicht erzeugen,
           Pool erzeugen
  Parameter: Wochentag, Datum, Ende der Fruehschicht, Ende der Spaetschicht
  vorher : -
  nachher: Tag ist erzeugt, Liste der Schichten enthlt 2 Eintrge

Besetzen
  Auftrag: alle Schichten werden mit passenden Mitarbeitern besetzt
  vorher : ist erzeugt
  nachher: alle Schichten neu besetzt
  Rckgabewerte: true = erfolgreich besetzt
                 false = keine Besetzung mglich

NeuerArbeitsplatz
  Auftrag: erzeugt fr den neuen Arbeitsplatz 1 neue Frh- und 1 neue Sptschicht
  Parameter: Name des Arbeitsplatzes, Ende der Fruehschicht, Ende der Spaetschicht
  vorher : -
  nachher: es gibt 2 neue Schichten

Set...
  Auftrag: Attribut schreiben
  vorher : Tag ist vorhanden
  nachher: Attribut ist gesetzt

GetAnzahlAP
  Auftrag: gibt die Anzahl der verwalteten Arbeitspltze zurck
  vorher : Tag ist vorhanden.
  nachher: -

Get...
  Auftrag: Attribut aus dem Objekt lesen
  vorher : Tag ist vorhanden
  nachher: -

Free
  Auftrag: loescht den Tag, die Schichten und den pool
  vorher : -
  nachher: -

----------------------------------------------------------------------- *)

IMPLEMENTATION
(* ==================================================================== *)

constructor TTag.Create;
(* -------------------------------------------------------------------- *)
begin
   inherited Create;
   Tag := WTAg;
   Datum := dat;
   anzahlap := 1;
   anzBueffet := 1;
   anzKueche := 1;
   anzSpuele := 1;

   Schichten := TListe.Create;

   NeuerArbeitsplatz(Arbeitsplatz, EndeFS, EndeSS);

   pool := TPool.Create;
end;

destructor TTag.Free;
(* -------------------------------------------------------------------- *)
begin
     Schichten.Free;
     pool.Free;
end;

function TTag.Besetzen;
(* -------------------------------------------------------------------- *)
var i : integer;
    schicht : TSchicht;
begin
     Result := true;

     pool.RemoveAll;
     pool.Auffuellen(MAListe, Tag);
     Schichten.First;
     schicht := TSchicht.Create('',0,0,0);
     for i:= 1 to Schichten.GetLen do
     begin
          schicht := Schichten.GetElement;
          if not schicht.Besetzen(Tag, pool, MAListe) then
               result := false;  // bricht nicht sofort ab, versucht restliche Schichten zu besetzen

          Schichten.PutElement(schicht);
          Schichten.Next;
     end;
end;

function TTag.GetWochentag;
(* -------------------------------------------------------------------- *)
begin
  result := Tag;
end;

function TTag.GetDatum;
(* -------------------------------------------------------------------- *)
begin
  result := Datum;
end;

function TTag.GetAnzahlAP;
(* -------------------------------------------------------------------- *)
begin
  result := AnzahlAP;
end;

function TTag.GetSchichten;
(* -------------------------------------------------------------------- *)
begin
  result := Schichten;
end;

procedure TTag.NeuerArbeitsplatz;
(* -------------------------------------------------------------------- *)
var fs,ss : TSchicht;
    s : string;
begin
  if  (Arbeitsplatz =  'Kche')
  then
  begin
       s := Arbeitsplatz + ' ' + IntToSTr(anzkueche);
       inc(anzkueche);
  end;

  if  (Arbeitsplatz =  'Bffet')
  then
  begin
       s := Arbeitsplatz + ' ' + IntToSTr(anzbueffet);
       inc(anzbueffet);
  end;

  if  (Arbeitsplatz =  'Sple')
  then
  begin
       s := Arbeitsplatz + ' ' + IntToSTr(anzspuele);
       inc(anzspuele);
  end;


   fs := TSchicht.Create(s, 1, EndeFS, 0);
   Schichten.Append(fs);
   ss := TSchicht.Create(s, EndeFS+1, EndeSS, 1);
   Schichten.Append(ss);
   inc(AnzahlAP);
end;

procedure TTag.SetWochentag;
(* -------------------------------------------------------------------- *)
begin
  Tag := WTAg;
end;


procedure TTag.SetDatum;
(* -------------------------------------------------------------------- *)
begin
  Datum := d;
end;

procedure TTag.InDateiSchreiben (var Fs : TFileStream);
(* -------------------------------------------------------------------- *)
begin
  Fs.write(Tag, sizeof(Tag));
  Fs.write(Datum, sizeof(Datum));
  Fs.write(AnzahlAP, sizeof(AnzahlAP));

  //Schichten.InDateiSchreiben(fs);
end;


procedure TTag.AusDateiLesen (var Fs : TFileStream) ;
(* -------------------------------------------------------------------- *)
begin
  Fs.read(Tag, sizeof(Tag));
  Fs.read(Datum, sizeof(Datum));
  Fs.read(AnzahlAP, sizeof(AnzahlAP));

  //Schichten.AusDateiLesen(fs);
end;

end.
